static char rcsid[] ="$Id: rftp.c,v 1.8 1997/11/29 23:42:08 kline Exp kline $";

#include <stdio.h>
#include <unistd.h>
#include <signal.h>
#include <sys/param.h>
#include <errno.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <time.h>
#include <unistd.h>
#include "msg.h" 
#include "rftp.h" 

static short bitftp = 0, ftpmail = 0;  /* which retrieval system */
static short setup  = 0,setuperr = 0;  /* is our environment set */
main(argc, argv)
register short int argc;
register char **argv;
{
  extern char *version_string;
  register short  childpid, fd, i;
  void start_action(), create_temp_ftp_dir(), get_environment();
  char *str_bitftp  = "is waiting for receipt of your files from bitftp";
  char *str_ftpmail = "is waiting for receipt of your files from ftpmail";

  msg_init();  /* pull in Kjetil Homme's debug library */

  if (argc != 2)
  {
    usage(argv[0]);
    exit(1);
  }
  for (i = 1; i < argc; i++)
  {
    if (argv[i][0] == '-')
    {
      switch (argv[i][1])
      {

      case 'V':
	(void)version();
	exit(0);
      case 'e':
        setup++;
        break;

      case 'b':
        bitftp++;
        break;

      case 'f':
        ftpmail++;
        break;

      default:
        usage(argv[0]);
        exit(1);
      }
    }
  }
  if (bitftp)
     fprintf(stdout, "%s %s\n", argv[0], str_bitftp);
  else if (ftpmail)
     fprintf(stdout, "%s %s\n", argv[0], str_ftpmail);


  checkbins();
  /*
   * get environment variables to create and check files and directories;
   * create a temporary ftp directory and chdir to it
   * for the duration.  set umask.  then enter loop.
   */
 (void)get_environment();
 (void)create_temp_ftp_dir();
 if (setuperr != 0)
 {
    fprintf(stderr, "There are probably configuration errors. Exiting.\n");
    exit(1);
 }
 /*
  * otherwise, no errors on startup, so proceed....
  */

 (void)chdir (ftptmp);
 (void)umask(0);

  /*  
   *  fire off daemon::::: 
   */
  if ((childpid = fork()) < 0)
    err_exit("main", "fork: can't fork");
  else if (childpid > 0)
  {
    exit(0); /* _this_ process */
  }

  /*
   *  1st child process.
   *  disassociate from controlling terminal and proc group.
   *  ensure that the proc can't reaqire a new controlling terminal
   */

#ifdef BSD
  if (setpgrp(0, getpid() ) == -1)
    err_exit("main", "setpgrp: can't change process group");
  if ((fd = open("/dev/tty", O_RDWR)) >=  0)
  {
    ioctl (fd, TIOCNOTTY, (char *) NULL); /* lose contrling tty */
    (void)close (fd);
  }
#else /* #if SVRX */
  (void)setpgrp();  /* change proc group and lose controlling tty  */
#endif




 (void)start_action ();

  /*  Close any open file descriptors  */
  for (fd = 0; fd < NOFILE; fd++)
    (void)close (fd);

  exit(0);  /* successful exit */
}



/*
 *  start of business: loop until one exit condition or another.
 */
void
start_action ()
{
  boolean ftp_file_reconstruct();
  boolean finished = false;

  while (! finished)
  {
    finished = (short)ftp_file_reconstruct();
    /*(void)sleep (QUARTER_HOUR);     /*  every TIME_SLICE check mbox */
    sleep (30);
  }

}




boolean
ftp_file_reconstruct()
{

  boolean has_from_line(), request_received(), 
      subject_results(), success_indicated();
  register short int fcount = 0, last, part; 
  boolean do_reconstruct = false;
  char *complete_file(), *return_filename(), *return_ftphost();
  static char buf[64],  fname[64],ftphost[64], 
  *site,*complete_results;
  char *file_retrieved;

  clear(buf);
  clear(fname);
  clear(ftphost);
  if (has_from_line())  /* valid mail in spooler file */
  {
    if (subject_results())        /* "Subject: " results indicate the         */
    {                             /*  ftp-mail program has returned something */
      if (success_indicated())
      {
        if ((site = return_ftphost()))  /*  We have  the site/host name */
        {
	  MSG(TRACE,1, ("return_ftphost retd site = [%s]\n", site));
          strcpy(ftphost, site);
          do_reconstruct = 1;
        }
        else
        {
          (void)sprintf(buf, "Subject: results valid, but no site retrieved!");
          mail_msg(buf, _F_FTP_FILE,  DO_EXIT);
        }
        if ((file_retrieved = return_filename()))/*The file that we requested */
        {
	  MSG(TRACE,1,("ftp_file_reconstruct: file_retrieved = [%s]\n", 
	      file_retrieved));
          strcpy (fname, file_retrieved); 
          do_reconstruct = 1;
        }
        else
        {
          (void)sprintf(buf, "Subject: results valid, but no file retrieved!");
          mail_msg(buf, _F_FTP_FILE,  DO_EXIT);
        }
      }
      else
      {
        /*
          * initial failure: mail a message and exit
          */
        (void)sprintf(buf, "bitftp reports: No such file or host");
          mail_msg(buf, _F_FTP_FILE,  DO_EXIT);
      }
    }
    else
    {
      /* 
       ** subject_results file not here yet: loop
       */
      do_reconstruct = 0;
    }
  }
  else
  {
         /*
          * Spool mail is either garbaged or zero-len: loop
          */
    do_reconstruct = 0;
  }


     /*
      *  if we have both the site and the filename, then attempt to
      * get the number of the LAST file, then copy files from the spool mail
      * directory to a tmp directory, and put everything together.
      * if failure, keep on looping... .
      *   if don't have site and filename, return to looping... .
      */
   MSG(TRACE, 1, ("ftp_file_reconstruct: fname = [%s], ftphost = [%s]\n", 
      fname, ftphost));

  /* no need to bother going any further unless things are okay */
  if (do_reconstruct)
  {
    if (bitftp)
    {
      if ((last = get_bitftp_last_part(fname, ftphost)) > 0)
      {
        MSG(TRACE,2,("do_reconstruct (bitftp): number of files: %d\r\n",last));
        for (part = 1; part < last+1; part++)
        {
          fcount += get_one_to_last(part, fname, ftphost);
        }

          /*
           *  BE SURE all parts are here before we try to glue it together
           */
        if (last != fcount)
        {
          sprintf(buf, "Missing (%d) file(s): continuing to wait\n", part-last);
          mail_msg(buf, _F_FTP_FILE,  DO_NOT_EXIT);
          do_reconstruct = 0;   /*  (*sigh*) incomplete: keep on trying... */
        }
        else
        {
          if ((complete_results = complete_file(fname)) != NULL)
          {
            mail_msg(complete_results, _F_FTP_FILE,
                DO_EXIT);
          }
          else
          {
            do_reconstruct = 0;   /*  (*sigh*) incomplete: keep on trying... */
          }
        }
      }
      else
      {
        /*
         * the ``last'' filenumber is obviously invalid since it must at
         * least be "1"
         */
        sprintf(buf, "Invalid last file number = (%d)! Bailing out\n",
	   last);
          mail_msg(buf, _F_FTP_FILE,  DO_EXIT);
      }
    }
    else if (ftpmail)
    {
      /*
       *  the DEC ftpmail server differs signifigantly
       */
      if ((last = get_last_part(fname, ftphost)) != 0)
      {
        printf("do_reconstruct (ftpmail): number of files: %d\r\n",last);
        MSG(TRACE,2,("do_reconstruct (ftpmail): number of files: %d\r\n",last));
        for (part = 1; part < last+1; part++)
        {
          fcount += f_get_one_to_last(part, fname, ftphost);
        }
        /*
         *  BE SURE all parts are here before we try to glue it together
         */
        if (last != fcount)
        {
          sprintf(buf, "Missing (%d) file(s): continuing to wait\n", part-last);
          mail_msg(buf, _F_FTP_FILE,  DO_NOT_EXIT);
          do_reconstruct = 0;
        }
        else
        {
          if ((complete_results = complete_file(fname)) != NULL)
          {
            mail_msg(complete_results, _F_COMPLETE, DO_EXIT);
          }
          else
          {
            do_reconstruct = 0;   /*  (*sigh*) incomplete: keep on trying... */
          }
        }
      }
      else
      {
        /*
         * the ``last'' filenumber is obviously invalid since it must at
         * least be "1"
         */
        sprintf(buf, "Invalid last file number! last = (%d). Bailing out\n",
	   last);
        mail_msg(buf, _F_FTP_FILE,  DO_EXIT);
      }
    }
  }
  return(do_reconstruct);
}



/*
 *  determine that the spool directory 
 *  file contents have a valid mail "From" line.
 *  if not, return loop indications.
 */
boolean
has_from_line()
{

  boolean is_from_line(), from_line = false;
  char buf[64], line[LBUF];

  if ((mfp = fopen(spooldir, "r")) == NULL)
  {
    sprintf(buf, "Cannot open mail spool file. Given as %s. Bailing.\n",
      spooldir);
    /* bummer: mail spooldir not configured: Quit now.  */
    mail_msg(buf, _F_HAS_FROM, DO_EXIT);
  }
  else
  {
    while (fgets(line, sizeof(line), mfp) != NULL)
    {
      if (is_from_line(line))
      {
        from_line = true;  /* at least some valid mail */
        break;
      }
    }
  }
  (void)fclose (mfp);
  return (from_line);
}


/*
 * do we find a valid "Subject: <>" entry for the mail server?
 */
boolean
subject_results()
{
  boolean subject_request_results = false;
  char buf[64], line[LBUF];
  if ((mfp = fopen(spooldir, "r")) == NULL)
  {
    sprintf(buf, "Cannot open %s\n", spooldir);
    mail_msg(buf, _F_SUBJECT, DO_NOT_EXIT);
  }
  else
  {
    while (fgets(line, sizeof(line), mfp) != NULL)
    {
      if (bitftp)
      {
        if(!strncmp(B_SUBJECT_RESULTS_STRING,line,strlen(B_SUBJECT_RESULTS_STRING)))
        {
          subject_request_results = true;
          break;
        }
      }
      else if (ftpmail)
      {
        if(!strncmp(SUBJECT_RESULTS_STRING,line,strlen(SUBJECT_RESULTS_STRING)))
        {
          subject_request_results = true;
          break;
        }
      }
    }
  }
  (void)fclose (mfp);


  /*printf("subj: ret: (%d)\n", subject_request_results);*/
  return ((subject_request_results));
}



/*
 *  having gotten this far, retrieve the name of the original ftp archive
 *  file that is being mailed back.
 */
static char * 
return_filename()
{

  boolean is_from_line(),is_retr_line();
  char buf[64], line[LBUF], *reduce_string();
  static char filename[64], *fn;

  clear (filename);
  if ((mfp = fopen(spooldir, "r")) == NULL)
  {
    sprintf(buf, "Cannot open %s\n", spooldir);
    mail_msg(buf, _F_RETURN_FILE, DO_NOT_EXIT);
  }
  else
  {
    while (fgets(line, sizeof(line), mfp) != NULL)
    {
      if (is_from_line(line))
      {
        /* There *are* From lines; at least one... */
        while (fgets(line, sizeof(line), mfp) != NULL)
        {
          if (bitftp)
          {
            if (!strncmp(B_RETR_STRING,line,strlen(B_RETR_STRING) ))
            {
              fn = reduce_string(line, strlen(B_RETR_STRING));

              fn[strlen(fn)-1] = '\0'; /* axe newline */

	      MSG(TRACE, 3, ("filename fn after axeing newline = [%s]\n", fn));
	      strcpy(filename, fn); /* save array */
              (void)fclose (mfp);
              return (filename);
            }
          }
          else if (ftpmail)
          {
            if (!strncmp(RETR_STRING,line,strlen(RETR_STRING) ))
            {
              /* 
               * differs from bitftp version
               */
              fn = reduce_string(line, strlen(RETR_STRING));
                fn[strlen(fn)-1] = '\0'; /* axe newline */
                (void)fclose (mfp);
	      printf ("filename fn after axeing newline = [%s]\n", fn);
	      MSG(TRACE, 3, ("filename fn after axeing newline = [%s]\n", fn));
	      printf("rett filename = fn = [%s]\n", fn);
		 strcpy(filename, fn); /* save array */
                return (filename);
            }
          }
        } /* endwhile */
      }
    } /* endwhile */
  }

  (void)fclose (mfp);
  return (NULL);
}



/*
 *  from the mail file telling that ftpmail/bitftp has connected to the
 *  requested site, get that site/host name.
 */
static char *
return_ftphost()
{

  boolean is_from_line();
  char buf[64], line[LBUF], line2[LBUF];
  char *return_to_symbol(), *reduce_string();
  FILE *mfp;
  static char *host;

  if ((mfp = fopen(spooldir, "r")) == NULL)
  {
    sprintf(buf, "Cannot open %s\n", spooldir);
    mail_msg(buf, _F_BGET_LAST, DO_NOT_EXIT);
  }
  else
  {
    while (fgets(line, sizeof(line), mfp) != NULL)
    {
      if (is_from_line(line))
      {
        while (fgets(line2, sizeof(line2), mfp) != NULL)
        {
          if (ftpmail)
          {
            if (!strncmp(CONNECTING_STRING,line2,strlen(CONNECTING_STRING) ))
            {
              host = reduce_string(line2, strlen(CONNECTING_STRING));
              host[strlen(host)-1] = '\0'; /* axe newline */
              host = return_to_symbol(host, ' ');
              (void)fclose (mfp);
              printf("\r\nReturning ftpmail host: %s\r\n", host);
              MSG(TRACE, 2,("\r\nReturning ftpmail host: %s\r\n", host));
              return (host);
            }
          }
          else if (bitftp)
          {
            if (!strncmp(B_CONNECTING_STRING,line2,strlen(B_CONNECTING_STRING)))
            {
              host = reduce_string(line2, strlen(B_CONNECTING_STRING));
              host[strlen(host)-1] = '\0'; /* axe newline */
              host = return_to_symbol(host, ' ');
              (void)fclose (mfp);
              return (host);
            }
          }
        }   /* endwhile */
      }
    }   /* endwhile */
  }

  (void)fclose (mfp);
  return (NULL);
}




/*
 * for the bitftp server, get the number of files being mailed to us.
 */
static int
get_bitftp_last_part(char *filename, char *site)
{

  boolean is_from_line(), match();
  char buf[64], line[LBUF]; 
  char *lastpart, *remainder_after_substring(), 
  *return_to_symbol();
  register int lastpartnumber = 0;

  MSG(TRACE,2,("get_bitftp_last_part(): filename = [%s], site = [%s]\n", 
    filename, site));
  clear(buf)
  clear(line);
  if ((mfp = fopen(spooldir, "r")) == NULL)
  {
    sprintf(buf, "Cannot open %s\n", spooldir);
    mail_msg(buf, _F_BGET_LAST, DO_NOT_EXIT);
  }
  else
  {
    while (fgets(line, LBUF, mfp) != NULL)
    {
      if (is_from_line(line))
      {
        while (fgets(line, LBUF, mfp) != NULL)
        {
          if(!strncmp(SUBJECT_STRING,line,strlen(SUBJECT_STRING)))
          {
            if (match(line, B_PART_STRING) && match(line, B_LAST_STRING)
                && match(line, filename))
            {
              /*
               * this is the LAST file for this site 
               * So, get the part NNN
               */
              line[strlen(line)-1] = '\0'; /* axe newline */
              lastpart = remainder_after_substring(line, B_LAST_STRING);
              lastpart =return_to_symbol(lastpart, ' ');
              lastpartnumber = atoi(lastpart);
              MSG(TRACE, 2,( "lastpart = [%s], lastpartnumber = (%d)\n", 
                            lastpart , lastpartnumber));

              (void)fclose (mfp);
              return (lastpartnumber);
            }
          }
        }
      }
    }
  }
  (void)fclose (mfp);
  return (lastpartnumber);  /* failure */
}


/*
 * read each part, from the first to the last, in order, from the spooler
 */
static int
get_one_to_last(register int part, register char *file, register char *site)
{

  boolean is_from_line(), match();
  char buf[64], line[LBUF];
  void outfile();
  short returned = 0;

  if ((mfp = fopen(spooldir, "r")) == NULL)
  {
    sprintf(buf, "Cannot open %s\n", spooldir);
    mail_msg(buf, _F_GET_ONE, DO_NOT_EXIT);
  }
  else
  {
    while (fgets(line, LBUF, mfp) != NULL)
    {
      if (is_from_line (line))
      {
        while (fgets(line, LBUF, mfp) != NULL)
        {
          if(!strncmp(SUBJECT_STRING,line,strlen(SUBJECT_STRING)))
          {
            if (match(line,B_PART_STRING) && match(line,file))
            {
              line[strlen(line)-1] = '\0'; /* axe newline: just _because_ */
              if ((short)get_bftp_partnumber(line) == part)
              {
                (void)fprintf(stdout,"Grabbing part %d of file [%s]\r\n",
                    part, file);
                outfile (part, mfp);
                returned = 1;
              }
            }
          }
        }
      }
    }
  }
  (void)fclose (mfp);
  return (returned);
}



/*
 * get partnumber for ftpmail-style files.
 */
static int
get_ftpmail_part(register char *s )
{
  register char *pn, *tmp;
  char *reduce_string();
  register short int dcount = 0;

  pn =tmp = reduce_string(s,strlen(SUBPART_STRING));
  while (*pn)   /* get exact count of digits: may be 2; or 3 */
  {
    if (isdigit(*pn)) dcount++;
    pn++;
  }
  pn = left(tmp, dcount);
  printf("get_ftpmail_part(): s = [%s], pn = [%s]\n", s, pn);

  MSG(TRACE,2, ("get_ftpmail_part(): s = [%s], pn = [%s]\n", s, pn));
  return (atoi(pn));
}


/*
 *  fast check for "^From " string plus partial check for date string.
 */
boolean
is_from_line(register char *s )
{
  boolean is_date(), ret_bval = false;

  if ((s[0]=='F'&&s[1]=='r'&& s[2]=='o'&&s[3]=='m'&& s[4]== ' ') && is_date(s))
    ret_bval = true;
  return (ret_bval);
}


/*
 * check for day and month in `date` fmt.
 */
boolean
is_date(register char *s)
{
  boolean match(), ret_bval = false;
  register short int i,j;

  for (i = 0; i < 7; i++)
  {
    if (match(s, day_string[i]))
    {
      for (j = 0; j < 12; j++)
      {
        if (match(s, month_string[j]))
        {
          ret_bval = true;
          break;
        }
      }
    }
  }
  return (ret_bval);
}


/*
 *  write the output files, temporarily....
 */
void
outfile(register int part, register FILE *mfp)
{
  boolean is_from_line();
  register FILE *tmp_fp;
  char buf[64], cnum[4], line[LBUF], Ofilename[64];

  if (part < 10) (void)sprintf(cnum, "00%d", part);
  else if (part >= 10 && part < 100) (void)sprintf(cnum, "0%d", part);
  else if (part >= 100 && part < 1000) (void)sprintf(cnum, "%d", part);
  (void)sprintf(Ofilename, "%s/%s%s", ftptmp, PREFIX, cnum);

  if ((tmp_fp = fopen(Ofilename, "w")) == NULL)
  {
    sprintf(buf, "Cannot open %s\n", Ofilename);
    mail_msg(buf, _F_OUTFILE, DO_EXIT);
  }

  while (fgets(line, LBUF, mfp) != NULL)
  {
    if (is_from_line(line))
    {
      (void)fclose (tmp_fp);
      return;
    }
    (void)fputs(line, tmp_fp);
  }
  /*
      *  if "part NNN" is the last file in the SPOOL, with *no* following "From"
      *  lines....
      */
  (void)fclose (tmp_fp);
}


/*
 *  all thes "part" have likely been retrieved; now put everything together.
 */
static char *
complete_file(register char *file)
{
  char asterisk = '*', cmd[128], uncompressed[32];
  static char *success_prob = "Success probable.  Check out file";
  static char buf[128];  /* all-purpose buffer ... */


  clear (buf);
  if (access(ftp_received, F_OK) == 0 && access(ftp_received, 2) != 0)
  { 
    sprintf(buf, 
      "File %s exists as unwritable;\ncheck and remove before re-attempting\n",
	ftp_received);
    mail_msg(buf, _F_COMPLETE, DO_EXIT);
  }

  (void)sprintf(cmd,"%s %s/%s%c | %s",UUCAT,ftptmp,PREFIX,asterisk,UUDECODE);
  MSG(TRACE,3,("complete_file: UUCAT cmd = [%s]\n", cmd));

  if (system (cmd) != SYSTEM_EXIT_OKAY)
  {
    (void)sprintf(buf, "system(cmd) failed with cmd: [%s]\n", cmd);
  }
  else
  {
    if (access(ftp_received, F_OK) == 0)
    {
      (void)sprintf(cmd, "%s %s %s/%s", MV, ftp_received, ftptmp, file);
      printf("complete_file: MV cmd = [%s]\n", cmd);

      MSG(TRACE,3,("complete_file: MV cmd = [%s]\n", cmd));
      if (system (cmd) != SYSTEM_EXIT_OKAY)
      {
        (void)sprintf(buf, "system(cmd) failed with cmd: [%s]\n", cmd);
      }
    }
    else
    {
      (void)sprintf(buf, "Error:  access([%s], F_OK) failed.\n", ftp_received);
    }

    /*
     * cleanup the _p.* files and send mail message that file.Z has 
     * probably arrived safely.
     */
    (void)sprintf(cmd, "%s %s/%s%c", RM, ftptmp, PREFIX, asterisk);
    if (system (cmd) != SYSTEM_EXIT_OKAY)
      (void)sprintf(buf, "system(cmd) failed with cmd: [%s]\n", cmd);
    else
    {
      (void)sprintf(buf, "%s %s/%s", success_prob,  ftptmp, file);
    }
  }
  return (buf);
}

/*
 * person to NOTIFY of detected results
 */
void
mail_msg(register char *msg, register char *subject, register int leave)
{
  char cmd[LBUF], *dash_s ="-s", pipe='|';

  (void)sprintf(cmd,"%s \"%s\" %c %s %s \"%s\" %s",
     B_ECHO,msg,pipe,MAILER,dash_s ,subject,NOTIFY);
  MSG(TRACE, 2,("debug: mail_msg(): cmd = [%s]\n", cmd));
  if (system (cmd) != SYSTEM_EXIT_OKAY)
  {
     (void)fprintf(stderr, "mail_msg(): `system(cmd)' failed! Exiting");
     exit(1);
  }
  if (leave)
     exit(1);
}





/*
 *  for bitftp server
 */
static int
get_bftp_partnumber(register char *line)
{
  char *pnum, *remainder_after_substring(), *return_to_symbol();
  register short int partnumber;
  pnum = remainder_after_substring(line, B_PART_STRING);
  pnum = return_to_symbol(pnum, ' ');
  partnumber = atoi(pnum);
  return ((int)partnumber);
}

/*
 *  for ftpmail server, retrieve the number of files being returned.
 */ 
static int
get_last_part(register char *file, register char *site)
{

  boolean is_from_line(), match();
  char buf[64], line[LBUF];
  register int lastpartnumber = 0;

  if ((mfp = fopen(spooldir, "r")) == NULL)
  {
    sprintf(buf, "Cannot open %s\n", spooldir);
    mail_msg(buf, _F_GET_LAST, DO_NOT_EXIT);
  }
  else
  {
    while (fgets(line, LBUF, mfp) != NULL)
    {
      if (is_from_line(line))
      {
        while (fgets(line, LBUF, mfp) != NULL)
        {
          if(!strncmp(SUBJECT_STRING,line,strlen(SUBJECT_STRING)))
          {
            if (match(line, LAST_STRING))
            {
	      printf("%s: line = [%s], site = [%s], file = [%s]\n",
		 _F_GET_LAST, line, site, file);

	      MSG(TRACE,2,("%s: line = [%s], site = [%s], file = [%s]\n",
		 _F_GET_LAST, line, site, file));
              if (match(line, site) && match(line, file))
              {
                /*
                 * this is the LAST file for this site 
                 * So, get the part NNN
                 */
                lastpartnumber = get_ftpmail_part(line);
		printf("%s: lastpartnumber = (%d)\n", 
		  _F_GET_LAST, lastpartnumber);
              }
            }
          }
        }
      }
    }
  }
  (void)fclose (mfp);
  return (lastpartnumber);
}

void
get_environment()
{
  char buf[64], *getenv();

  if ((home = getenv(HOME)) == NULL)
  {
    sprintf(buf, "HOME environment variable not set");
    mail_msg(buf, _F_GET_ENV, DO_EXIT);
  }
  if ((user = getenv(USER)) == NULL && (user = getenv(LOGNAME)) == NULL)
  {
    sprintf(buf, "USER environment variable not set");
    mail_msg(buf, _F_GET_ENV, DO_EXIT);
  }
  sprintf(spooldir, "%s%s", SPOOL_MAIL, user);
  if (setup)
  {
	  if (access(spooldir, F_OK) == 0) 
	  { 
             fprintf(stdout, "Mail spool file is [%s]\n", spooldir);
	  }
	  else
	  {
            fprintf(stdout, "Mail spool file is [%s] and is non-existant\7\n", 
		spooldir);
	    setuperr++;
	  }
  }
  MSG(TRACE, 2, ("get_environment: spooldir = [%s]\n", spooldir));


}
void
create_temp_ftp_dir()
{

  char buf[64];

  (void)sprintf(ftptmp, "%s/%s", home, H_FTPTMP);
  if (mkdir (ftptmp, F_MODE) < 0)
  {
    if (errno != EEXIST)  /* if it already exists, okay, otherwise exit */
    {
      sprintf(buf, "Cannot create %s\n", ftptmp);
      mail_msg(buf, _F_CREATE, DO_EXIT);
    }
  }
  /* string with "$HOME/$H_FTPTMP/ftp_received": for gathering p.NNN into */
  (void)sprintf(ftp_received, "%s/%s", ftptmp, H_FTPMAIL);

  MSG(TRACE,3,("ftp_received file for ftpmail is [%s]\n", ftp_received));
  if (setup)
  {
     if (access(spooldir, F_OK) == 0) 
     { 
       fprintf(stdout, "Temporary $HOME/ftptmp directory is %s\n", ftptmp);
     }
     else
     {
       fprintf(stdout,"Temporary $HOME/ftptmp directory %s was not created.\n",
	  ftptmp);
       fprintf(stderr,"Check your /* CONFIGURATION */ options in rftp.h\7\n\n");
       setuperr++;
     }
  }

}
static int
f_get_one_to_last(register int part, register char *file, register char *site)
{

  boolean is_from_line(), match();
  char buf[64], line[LBUF];
  short returned = 0;
  void outfile();

  if ((mfp = fopen(spooldir, "r")) == NULL)
  {
    sprintf(buf, "Cannot open %s\n", spooldir);
    mail_msg(buf, _F_F_GET_ONE,  DO_NOT_EXIT);
  }
  else
  {
    while (fgets(line, LBUF, mfp) != NULL)
    {
      if (is_from_line (line))
      {
        while (fgets(line, LBUF, mfp) != NULL)
        {
          if(!strncmp(SUBJECT_STRING,line,strlen(SUBJECT_STRING)))
          {
            if (match(line,PART_STRING) &&match(line,site) &&match(line,file))
            {
              if (get_ftpmail_part(line) == part)
              {
                (void)fprintf(stdout,"Grabbing part %d of file [%s]\r\n",
                    part, file);
                outfile (part, mfp);
                returned = 1;
              }
            }
          }
        }
      }
    }
  }
  (void)fclose (mfp);
  return (returned);
}
/*
 * give user a clue
 */
void
usage(s)
register char *s;
{
  (void) fprintf(stderr, "Usage, ``%s: [-b] [-f]''\n", s);
}
boolean
success_indicated()
{
  boolean success = false;
  char buf[64], line[LBUF];

  if ((mfp = fopen(spooldir, "r")) == NULL)
  {
    sprintf(buf, "Cannot open %s\n", spooldir);
    mail_msg(buf, _F_SUCCESS, DO_EXIT);
  }
  else
  {
    while (fgets(line, sizeof(line), mfp) != NULL)
    {
      if (is_from_line(line))
      {
        while (fgets(line, sizeof(line), mfp) != NULL)
        {
          if (bitftp)
          {
            /*
                * make sure bitftp actually transferred <filename> back to 
                * pucc.princeton.edu.  if so, chances are good that the 
                * file is here...or en-route
                */
            if (!strncmp(SUCCESSFUL_TRANSFER,line,strlen(SUCCESSFUL_TRANSFER)))
            {
              success = true;
            }
            else if(!strncmp(ENOFILE,line,strlen(ENOFILE)))
            {
              success = false;
            }
            else if(!strncmp(ENOANON,line,strlen(ENOANON))) /* host not anon */
            {
              (void)fprintf(stderr, " %s\r\n\n",line);
              success = false;
            }
          }
          else if(ftpmail)/* considerable difference between ftpmail & bitftp*/
          {
            if (!strncmp(F_FAILURE,line,strlen(F_FAILURE) ))
            {
              (void)fprintf(stderr, " %s\r\n\n",line);
              success = false;
            }
            else
            {
              success = true;
            }
          }
        } /* endwhile */
      }
    } /* endwhile */
  }
  (void)fclose (mfp);
  MSG(TRACE,1,("success_indicated: success = (%d)\r\n", success));
  putchar('\n');     /* format output */
  return (success);
}


/*
 *  check for uudecode and uucat...
 */
checkbins()
{
  static char buf[64];
  int retval;

  if (retval = access(UUCAT, X_OK) < 0)
  {
	fprintf(stderr, "Don't find uucat executable at %s\nstrerror: %s\n", 
		UUCAT, strerror(errno) );
  }
  if (retval = access(UUDECODE, X_OK) < 0)
  {
	fprintf(stderr, "Don't find essential uudecode executable at %s\n", 
	  UUDECODE);
	fprintf(stderr, "strerror: %s\n", strerror(errno));
  }
  if (retval < 0)
  {
	clear (buf);
	(void)sprintf(buf, "Subject: missing one or more essential binaries.");
	mail_msg(buf, _F_CHECKBIN,  DO_EXIT);

  }
}

