static char rcsid[] ="$Id: rftp_utils.c,v 1.2 1993/11/01 04:09:51 gdk Exp gdk $";
#include <stdio.h>
#include <unistd.h>
#include <signal.h>
#include <sys/param.h>
#include <errno.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <time.h>
#include <unistd.h>
#include "rftp.h"
#include "msg.h"

/*
 * rftp_utils.c
 */

/*****************************************************************************
FUNCTION remainder_after_substring(s, sub_s)
Description: Returns a string (ptr thereto) following the sub-string 'sub_s'
eg: passed: s = "This is a test",  sub_s = " is ", this returns: "a test"

Returns: remainder of a string following sub_s, or NULL

Called by:  b_get_last_part(),  get_bftp_partnumber()
******************************************************************************/

char *
remainder_after_substring(s, sub_s)
char *s, *sub_s;
{
  while (*s)
  {
    while(*sub_s)
    {
      if (*s == *sub_s)
      {
        if (!strncmp(s, sub_s, strlen(sub_s) ))
        {
          s += strlen(sub_s); /* past sub_s */
          return (s);
        }
        s++;
        break;
      }
      s++;
    }
    s++;
  }
  return (NULL);
}

/*****************************************************************************
FUNCTION return_to_symbol(s)
Description: Returns a string (ptr thereto) up to the character 'sybmol'

Returns: string up to char `symbol'

Called by:  return_ftphost(), b_get_last_part(), get_bftp_partnumber()
******************************************************************************/


char *
return_to_symbol(s, symbol)
register char *s, symbol;
{
  register char *s1 = s;
  register short int i, len = 0;
  static char p[256];

  for(i=0;i<256;i++)p[i]='\0';
  while (*s)
  {
    if (*s != symbol)
      s++, len++;
      else
      break;
  }
  strncpy (p,s1, len);
  return (p);
}


#include <ctype.h>

/*****************************************************************************
FUNCTION
Description:

Returns:  the ``count'' leftmost bytes of ``source''

Called by:  get_partnumber()
******************************************************************************/

char *
left(source, count)
register char *source;
register int count;
{
  static char st[256]; /* Increase this as required */

  (void)strncpy(st, source, count);
  st[count] = 0; /* Shorten it. */
  return (st);  /* And return it. */
}

/*****************************************************************************
FUNCTION
Description:

Returns:  the ``count'' rightmost bytes of ``source''

Called by:  ret_endpath()
******************************************************************************/

char *
right(source, count)
register char *source;
register int count;
{
  static char st[256]; /* Increase this as required */
  while (*source && count)
    source++, count--;
  return (source);
}


/*****************************************************************************
FUNCTION ret_endpath(s, part)
Description:  Returns the filename at the end of the path.

Returns:  End-of-path file

Called by: 
******************************************************************************/

char *
ret_endpath(path)
register char *path;
{
  register char *s = path, *p = path;
  register int ccount = 0, slashcount = 0;
  static char *endpath;

  while (*path)
  {
    ccount++, path++;                /* step to end of  string;  */
    if (*path == '/') slashcount++;
    if (slashcount == get_nsymbols(p, '/'))
    {
      endpath = right(s, ccount);
      endpath++;  /* past last slash */
      return (endpath);
    }
  }
  return ((char*)0);
}


/*****************************************************************************
FUNCTION get_nsymbols(s, symbol)
Description:  Returns, in this case, the number of "/" characters in s

Returns:  (int)nsymbols

Called by:  ret_endpath()
******************************************************************************/
int
get_nsymbols(s, symbol)
register char *s, symbol;
{
  register short int nsymbols = 0;
  while (*s)
  {
    if (*s == symbol) nsymbols++;
    s++;
  }
  return ((int)nsymbols);
}

/*****************************************************************************
FUNCTION reduce_string(s, n)
Description:  Returns a string reduces by n bytes.  

Returns:  s

Called by:  return_filename(),  return_ftphost(), get_partnumber().
******************************************************************************/
char *
reduce_string(s, n)
register char *s;
register short int n;
{
  register short int i;

  for (i = 0; i < n; i++) s++;
  while (isspace(*s)) s++;  /* don't take any chance of having whitespace */
  MSG(TRACE, 3 , ("reduce_string returning: s = [%s] ", s));
  return (s);
}


/*
 * quick-enough pattern-matching function
 */
/*****************************************************************************
FUNCTION match(argument, pattern)
Description:  True if pattern is found in argument; else false

Returns:  boolean

Called by:  b_get_last_part() get_one_to_last() get_one_to_last(), is_date(),
      get_last_part()
******************************************************************************/
boolean
match(argument, pattern)
register char *argument, *pattern;
{
  register boolean ret_bval = false;
  register short int length = strlen(pattern);
  while ((short)strlen(argument) >= length)
  {
    if (!strncmp(argument++, pattern, length))
    {
      ret_bval = true;
      break;
    }
  }
  return (ret_bval);
}
err_exit(function, e_str, s)
register char *function, *e_str, *s;
{
  (void) fprintf(stderr, "In function [%s], this error:\n", function);
  if (s)
  {
    (void) fprintf(stderr, "[%s]", e_str);
    (void) fprintf(stderr, " [%s]\n", s);
  }
  else
    (void) fprintf(stderr, "[%s]\n", e_str);
  exit(1);
}

