static char rcsid[] ="$Id: uucat.c,v 1.1 1997/10/23 22:51:41 kline Exp kline $";
/* ==================== uucat.c ==================== */
/*  cc -o uucat -O uucat.c                           */
/*
   Modified by Don Phillips at Research Unlimited 05-Oct-91
      correctly handle short lines at end of file.

   uucat file1 file2 ... filen | uudecode
   or
   cat file1 file2 ... filen | uucat | uudecode
   or, if you already have concatenated them into one file
   uucat file | uudecode

*/

/* uucat.c */
#include <stdio.h>
#include <string.h>

#define TRUE	1
#define FALSE	0

#define LENGTH	150

extern void uuread();


void main(argc, argv)
register short int argc;
char *argv[];
{
  int error, argno;
  FILE *infile;

  if (argc < 2)
    uuread(stdin);
  else
  {
    error = FALSE;
    for (argno = 1; !error && argno < argc; argno++)
    {
      if ((infile = fopen(argv[argno], "r")) == NULL)
      {
	error = TRUE;
	fprintf(stderr, "uucat: Can't open '%s' for input.\n", argv[argno]);
      }
      else
      {
	uuread(infile);
	fclose(infile);
      }
    }
  }
}


void uuread(infile)
register FILE *infile;
{
  char *s2, *s1, *s0, *tmp_s;
  register short int length;
  static char s[3 * (LENGTH + 1)];
  static int echo_on = FALSE, started = FALSE, just_finished = FALSE;
  static int line_length = 0, lines_to_go = -1;

  s0 = s;
  s1 = s0 + (LENGTH + 1);
  s2 = s1 + (LENGTH + 1);

  s0[0] = s1[0] = s2[0] = '\0';  /* Clear strings */

  while (fgets(s0, LENGTH, infile) != NULL)
  {
    s0[LENGTH] = '\0';  /* Make sure string is terminated */

    if (just_finished)
    {
      if (strncmp(s0, "size ", 5) == 0)
      {
	fputs(s0, stdout);
	s0[0] = '\0';
      }
      just_finished = FALSE;
    }

    if (!started)
    {
      if (strncmp(s0, "begin ", 6) == 0)
      {
	started = echo_on = TRUE;
	line_length = 0;
	lines_to_go = -1;
      }
    }
    else  /* started */
    {
      length = strlen(s0);
      if (line_length == 0)
	line_length = length;

      if (echo_on)
      {
	if (s0[0] != 'M' || length != line_length)
	{
	  echo_on = FALSE;
	  lines_to_go = 2;	/* Lines to go before 'end' is expected */
	}
	else
	{
	  lines_to_go = -1;
	}
      }
      else  /* !echo_on */
      {
	if (s0[0] == 'M' && length == line_length)
	{
	  echo_on = TRUE;
	  lines_to_go = -1;
	}
	else
	{
	  lines_to_go--;
	}
      }
    }

    if (echo_on)
    {
      fputs(s0, stdout);
      s0[0] = '\0';
    }
    else if (strcmp(s0, "end\n") == 0)
    {
      if (lines_to_go < 0)
      {
	fprintf(stderr,"Warning: ");
	fprintf(stderr,"line buffer overflow. lines_to_go = %d\n",lines_to_go);
      }
      if (lines_to_go < 1)
      {
	fputs(s2, stdout);
      }
      if (lines_to_go < 2)
      {
	fputs(s1, stdout);
      }
      fputs(s0, stdout);
      lines_to_go = -1;		/* Done. Break off */
      just_finished = TRUE;
      started = FALSE;
    }

    tmp_s = s2;
    s2 = s1;
    s1 = s0;
    s0 = tmp_s;
  }
}
